@@ -688,3 +688,19 @@ sub AAT_write_lookup
 
 1;
 
+=head1 AUTHOR
+
+Jonathan Kew L<Jonathan_Kew@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
+
+=cut
+
