@@ -170,7 +170,7 @@ Device table for device specific adjustment of vertical advance
 
 =item XIdPlacement
 
-Horizontal placement metric id (for Multiple Master fonts - but that's all I know!)
+Horizontal placement metric id (for Multiple Master fonts - but that is all I know!)
 
 =item YIdPlacement
 
@@ -257,7 +257,7 @@ sub read_sub
         for ($i = 0; $i < $mcount; $i++)
         { push (@{$lookup->{'RULES'}}, [{'ACTION' =>
                                     [$self->read_value($count, $loc, $lookup, $fh)]}]); }
-        $self->{'ACTION_TYPE'} = 'v';
+        $lookup->{'ACTION_TYPE'} = 'v';
     } elsif ($type == 2 && $fmt == 1)
     {
         $lookup->{'VFMT'} = $count;
@@ -389,30 +389,29 @@ Outputs the subtable to the given filehandle
 
 sub out_sub
 {
-    my ($self, $fh, $main_lookup, $index) = @_;
+    my ($self, $fh, $main_lookup, $index, $ctables, $base) = @_;
     my ($type) = $main_lookup->{'TYPE'};
     my ($lookup) = $main_lookup->{'SUB'}[$index];
     my ($fmt) = $lookup->{'FORMAT'};
     my ($out, $r, $s, $t, $i, $j, $vfmt, $vfmt2, $loc1);
     my ($num) = $#{$lookup->{'RULES'}} + 1;
-    my ($ctables) = {};
     my ($mtables) = {};
     my (@reftables);
     
     if ($type == 1 && $fmt == 1)
     {
-        $out = pack('n2', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2));
+        $out = pack('n2', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2 + $base));
         $vfmt = $self->fmt_value($lookup->{'ADJUST'});
-        $out .= pack('n', $vfmt) . $self->out_value($lookup->{'ADJUST'}, $vfmt, $ctables, 6);
+        $out .= pack('n', $vfmt) . $self->out_value($lookup->{'ADJUST'}, $vfmt, $ctables, 6 + $base);
     } elsif ($type == 1 && $fmt == 2)
     {
         $vfmt = 0;
         foreach $r (@{$lookup->{'RULES'}})
         { $vfmt |= $self->fmt_value($r->[0]{'ACTION'}[0]); }
-        $out = pack('n4', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2),
+        $out = pack('n4', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2 + $base),
                             $vfmt, $#{$lookup->{'RULES'}} + 1);
         foreach $r (@{$lookup->{'RULES'}})
-        { $out .= $self->out_value($r->[0]{'ACTION'}[0], $vfmt, $ctables, length($out)); }
+        { $out .= $self->out_value($r->[0]{'ACTION'}[0], $vfmt, $ctables, length($out) + $base); }
     } elsif ($type == 2 && $fmt < 3)
     {
         $vfmt = 0;
@@ -430,7 +429,7 @@ sub out_sub
             # start PairPosFormat1 subtable
             $out = pack('n5', 
                         $fmt, 
-                        Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2),
+                        Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2 + $base),
                         $vfmt, 
                         $vfmt2, 
                         $#{$lookup->{'RULES'}} + 1); # PairSetCount
@@ -438,58 +437,65 @@ sub out_sub
             $off += length($out);
             $off += 2 * ($#{$lookup->{'RULES'}} + 1); # there will be PairSetCount offsets here
             my $pairsets = '';
+            my (%cache);
             foreach $r (@{$lookup->{'RULES'}}) # foreach PairSet table
             {
                 # write offset to this PairSet at end of PairPosFormat1 table
-                $out .= pack('n', $off);
-
-                # generate PairSet itself (using $off as eventual offset within PairPos subtable)
-                my $pairset = pack('n', $#{$r} + 1); # PairValueCount
-                foreach $t (@$r) # foreach PairValueRecord
+                if (defined $cache{"$r"})
+                { $out .= pack('n', $cache{"$r"}); }
+                else
                 {
-                    $pairset .= pack('n', $t->{'MATCH'}[0]); # SecondGlyph - MATCH has only one entry
-                    $pairset .= 
-                        $self->out_value($t->{'ACTION'}[0], $vfmt,  $ctables, $off + length($pairset));
-                    $pairset .= 
-                        $self->out_value($t->{'ACTION'}[1], $vfmt2, $ctables, $off + length($pairset));
+                    $out .= pack('n', $off);
+                    $cache{"$r"} = $off;
+
+                    # generate PairSet itself (using $off as eventual offset within PairPos subtable)
+                    my $pairset = pack('n', $#{$r} + 1); # PairValueCount
+                    foreach $t (@$r) # foreach PairValueRecord
+                    {
+                        $pairset .= pack('n', $t->{'MATCH'}[0]); # SecondGlyph - MATCH has only one entry
+                        $pairset .= 
+                            $self->out_value($t->{'ACTION'}[0], $vfmt,  $ctables, $off + length($pairset) + $base);
+                        $pairset .= 
+                            $self->out_value($t->{'ACTION'}[1], $vfmt2, $ctables, $off + length($pairset) + $base);
+                    }
+                    $off += length($pairset);
+                    $pairsets .= $pairset;
                 }
-                $off += length($pairset);
-                $pairsets .= $pairset;
             }
             $out .= $pairsets;
             die "internal error: PairPos size not as calculated" if (length($out) != $off);
         } else
         {
-            $out = pack('n8', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2),
+            $out = pack('n8', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2 + $base),
                             $vfmt, $vfmt2,
-                            Font::TTF::Ttopen::ref_cache($lookup->{'CLASS'}, $ctables, 1),
-                            Font::TTF::Ttopen::ref_cache($lookup->{'MATCH'}[0], $ctables, 1),
-                            $#{$lookup->{'RULES'}} + 1, $#{$lookup->{'RULES'}[0]} + 1);
+                            Font::TTF::Ttopen::ref_cache($lookup->{'CLASS'}, $ctables, 8 + $base),
+                            Font::TTF::Ttopen::ref_cache($lookup->{'MATCH'}[0], $ctables, 10 + $base),
+                            $lookup->{'CLASS'}{'max'} + 1, $lookup->{'MATCH'}[0]{'max'} + 1);
 
-            foreach $r (@{$lookup->{'RULES'}})
+            for ($i = 0; $i <= $lookup->{'CLASS'}{'max'}; $i++)
             {
-                foreach $t (@$r)
+                for ($j = 0; $j <= $lookup->{'MATCH'}[0]{'max'}; $j++)
                 {
-                    $out .= $self->out_value($t->{'ACTION'}[0], $vfmt, $ctables, length($out));
-                    $out .= $self->out_value($t->{'ACTION'}[1], $vfmt2, $ctables, length($out));
+                    $out .= $self->out_value($lookup->{'RULES'}[$i][$j]{'ACTION'}[0], $vfmt, $ctables, length($out) + $base);
+                    $out .= $self->out_value($lookup->{'RULES'}[$i][$j]{'ACTION'}[1], $vfmt2, $ctables, length($out) + $base);
                 }
             }
         }
     } elsif ($type == 3 && $fmt == 1)
     {
-        $out = pack('n3', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2),
+        $out = pack('n3', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 2 + $base),
                             $#{$lookup->{'RULES'}} + 1);
         foreach $r (@{$lookup->{'RULES'}})
         {
-            $out .= pack('n2', Font::TTF::Ttopen::ref_cache($r->[0]{'ACTION'}[0], $ctables, length($out)),
-                            Font::TTF::Ttopen::ref_cache($r->[0]{'ACTION'}[1], $ctables, length($out) + 2));
+            $out .= pack('n2', Font::TTF::Ttopen::ref_cache($r->[0]{'ACTION'}[0], $ctables, length($out) + $base),
+                            Font::TTF::Ttopen::ref_cache($r->[0]{'ACTION'}[1], $ctables, length($out) + 2 + $base));
         }
     } elsif ($type == 4 || $type == 5 || $type == 6)
     {
         my ($loc_off, $loc_t, $ltables);
         
-        $out = pack('n7', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'MATCH'}[0], $ctables, 2),
-                            Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 4),
+        $out = pack('n7', $fmt, Font::TTF::Ttopen::ref_cache($lookup->{'MATCH'}[0], $ctables, 2 + $base),
+                            Font::TTF::Ttopen::ref_cache($lookup->{'COVERAGE'}, $ctables, 4 + $base),
                             $#{$lookup->{'RULES'}[0][0]{'ACTION'}} + 1, 12, ($#{$lookup->{'MARKS'}} + 4) << 2,
                             $#{$lookup->{'MARKS'}} + 1);
         foreach $r (@{$lookup->{'MARKS'}})
@@ -524,11 +530,11 @@ sub out_sub
             push (@reftables, [$ltables, $loc_t]) if ($type == 5);
         }
         push (@reftables, [$ltables, $loc_t]) unless ($type == 5);
+        $out = Font::TTF::Ttopen::out_final($fh, $out, \@reftables, 1);
     } elsif ($type == 7 || $type == 8)
-    { $out = $self->out_context($lookup, $fh, $type - 2, $fmt, $ctables, $out, $num); }
-    push (@reftables, [$ctables, 0]);
-    Font::TTF::Ttopen::out_final($fh, $out, \@reftables);
-    $lookup;
+    { $out = $self->out_context($lookup, $fh, $type - 2, $fmt, $ctables, $out, $num, $base); }
+#    push (@reftables, [$ctables, 0]);
+    $out;
 }
             
 
@@ -564,7 +570,7 @@ sub read_value
     foreach $s (qw(XPlaDevice YPlaDevice XAdvDevice YAdvDevice))
     {
         if ($fmt & $flag)
-        { $res->{$s} = $self->read_delta(TTF_Unpack('S', substr($i++ << 1, 2)),
+        { $res->{$s} = $self->read_delta(TTF_Unpack('S', substr($dat, $i++ << 1, 2)),
                             $base, $lookup, $fh); }
         $flag <<= 1;
     }
@@ -580,7 +586,7 @@ sub read_value
 
 =head2 $t->read_delta($offset, $base, $lookup, $fh)
 
-Reads a delta (device table) at the given offset if it hasn't already been read.
+Reads a delta (device table) at the given offset if it hasn not already been read.
 Store the offset and item in the lookup cache ($lookup->{' CACHE'})
 
 =cut
@@ -604,7 +610,7 @@ sub read_delta
 
 =head2 $t->read_anchor($offset, $base, $lookup, $fh)
 
-Reads an Anchor table at the given offset if it hasn't already been read.
+Reads an Anchor table at the given offset if it has not already been read.
 
 =cut
 
@@ -683,13 +689,21 @@ sub out_value
     $out;
 }
 
+1;
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
-1;
-
