@@ -2,7 +2,7 @@ package Font::TTF::Glyph;
 
 =head1 NAME
 
-Font::TTF::Glyph - Holds a single glyph's information
+Font::TTF::Glyph - Holds a information for a single glyph
 
 =head1 DESCRIPTION
 
@@ -153,7 +153,7 @@ by calling the C<update> method whenever any of the glyph content changes.
 
 Location relative to the start of the glyf table. This variable is only active
 whilst the output process is going on. It is used to inform the location table
-where the glyph's location is, since the glyf table is output before the loca
+where the glyph is located, since the glyf table is output before the loca
 table due to alphabetical ordering.
 
 =item OUTLEN (P)
@@ -168,8 +168,8 @@ calling C<out> or C<out_dat>.
 =head2 Editing
 
 If you want to edit a glyph in some way, then you should read_dat the glyph, then
-make your changes and then update the glyph or set the $g->{' isdirty'} variable.
-It is the application's duty to ensure that the following instance variables are
+make your changes and then update the glyph or set the $g->{' isDirty'} variable.
+The application must ensure that the following instance variables are
 correct, from which update will calculate the rest, including the bounding box
 information.
 
@@ -240,7 +240,7 @@ sub new
 
 =head2 $g->read
 
-Reads the header component of the glyph (bounding box, etc.) and also the
+Reads the header component of the glyph (numberOfContours and bounding box) and also the
 glyph content, but into a data field rather than breaking it down into
 its constituent structures. Use read_dat for this.
 
@@ -252,7 +252,7 @@ sub read
     my ($fh) = $self->{' INFILE'};
     my ($dat);
 
-    return $self if $self->{' read'};
+    return $self if (defined $self->{' read'} && $self->{' read'} > 0);
     $self->{' read'} = 1;
     $fh->seek($self->{' LOC'} + $self->{' BASE'}, 0);
     $fh->read($self->{' DAT'}, $self->{' LEN'});
@@ -264,8 +264,7 @@ sub read
 =head2 $g->read_dat
 
 Reads the contents of the glyph (components and curves, etc.) from the memory
-store C<DAT> into structures within the object. Then, to indicate where the
-master form of the data is, it deletes the C<DAT> instance variable.
+store C<DAT> into structures within the object. 
 
 =cut
 
@@ -286,7 +285,9 @@ sub read_dat
         $max = 0;
         foreach (@{$self->{'endPoints'}})
         { $max = $_ if $_ > $max; }
-        $max++;
+#        print STDERR join(",", unpack('C*', $self->{" DAT"}));
+#        printf STDERR ("(%d,%d in %d=%d @ %d)", scalar @{$self->{'endPoints'}}, $max, length($dat), $self->{' LEN'}, $fp);
+        $max++ if (@{$self->{'endPoints'}});
         $self->{'numPoints'} = $max;
         $self->{'instLen'} = unpack("n", substr($dat, $fp));
         $self->{'hints'} = substr($dat, $fp + 2, $self->{'instLen'});
@@ -424,12 +425,12 @@ sub out_xml
 
     if ($context->{'addresses'}{$addr})
     {
-        $context->{'fh'}->printf("%s<glyph gid='%s' id_ref='%s'/>\n", $depth, $context->{'gid'}, $addr);
+        $context->{'fh'}->printf("%s<glyph gid='%s' id_ref='%s'/>\n", $depth, $context->{'gid'}, $context->{'addresses'}{$addr});
         return $self;
     }
     else
     {
-        $context->{'fh'}->printf("%s<glyph gid='%s' id='%s'>\n", $depth, $context->{'gid'}, $addr);
+        $context->{'fh'}->printf("%s<glyph gid='%s'>\n", $depth, $context->{'gid'});
     }
     
     $ndepth = $depth . $context->{'indent'};
@@ -478,23 +479,37 @@ sub XML_element
     $self;    
 }
 
+=head2 $g->dirty($val)
+
+This sets the dirty flag to the given value or 1 if no given value. It returns the
+value of the flag
+
+=cut
+
+sub dirty
+{
+    my ($self, $val) = @_;
+    my ($res) = $self->{' isDirty'};
+
+    $self->{' isDirty'} = defined $val ? $val : 1;
+    $res;
+}
 
 =head2 $g->update
 
 Generates a C<$self->{'DAT'}> from the internal structures, if the data has
 been read into structures in the first place. If you are building a glyph
-from scratch you will need to set the instance variable C<' read'> to 2 (or
-something > 1) for the update to work.
+from scratch you will need to set the instance variable C<' isDirty'>.
 
 =cut
 
 sub update
 {
     my ($self) = @_;
-    my ($dat, $loc, $len, $flag, $x, $y, $i, $comp, $num);
+    my ($dat, $loc, $len, $flag, $x, $y, $i, $comp, $num, @rflags, $repeat);
 
-    return $self unless (defined $self->{' read'} && $self->{' read'} > 1);
-    $self->update_bbox;
+    return $self unless ($self->{' isDirty'});
+    $self->read_dat->update_bbox;
     $self->{' DAT'} = TTF_Out_Fields($self, \%fields, 10);
     $num = $self->{'numberOfContours'};
     if ($num > 0)
@@ -503,6 +518,7 @@ sub update
         $len = $self->{'instLen'};
         $self->{' DAT'} .= pack("n", $len);
         $self->{' DAT'} .= pack("a" . $len, substr($self->{'hints'}, 0, $len)) if ($len > 0);
+        $repeat = 0;
         for ($i = 0; $i < $self->{'numPoints'}; $i++)
         {
             $flag = $self->{'flags'}[$i] & 1;
@@ -527,9 +543,28 @@ sub update
                 $flag |= 4;
                 $flag |= 32 if ($y >= 0);
             }
-            $self->{' DAT'} .= pack("C", $flag);                    # sorry no repeats
+            if ($i > 0 && $rflags[-1] == $flag && $repeat < 255)
+            {
+            	$repeat++;
+            } else
+            {
+            	if ($repeat)
+            	{
+            		$rflags[-1] |= 8;
+            		push @rflags, $repeat;
+            	}
+            	push @rflags, $flag;
+            	$repeat = 0;
+            } 
             $self->{'flags'}[$i] = $flag;
         }
+		# Add final repeat if needed, then pack up the flag bytes:
+    	if ($repeat)
+    	{
+    		$rflags[-1] |= 8;
+    		push @rflags, $repeat;
+    	}
+        $self->{' DAT'} .= pack("C*", @rflags);	
         for ($i = 0; $i < $self->{'numPoints'}; $i++)
         {
             $flag = $self->{'flags'}[$i];
@@ -568,7 +603,7 @@ sub update
                 {
                     if ($comp->{'scale'}[0] == $comp->{'scale'}[3])
                     { $flag |= 8 unless ($comp->{'scale'}[0] == 0
-                                    || abs(abs($comp->{'scale'}[0]) - 1.) < .001); }
+                                    || $comp->{'scale'}[0] == 1); }
                     else
                     { $flag |= 64; }
                 } else
@@ -604,7 +639,8 @@ sub update
             $self->{' DAT'} .= pack("a" . $len, substr($self->{'hints'}, 0, $len));
         }
     }
-    $self->{' DAT'} .= "\000" if (length($self->{' DAT'}) & 1);
+    my ($olen) = length($self->{' DAT'});
+    $self->{' DAT'} .= ("\000") x (4 - ($olen & 3)) if ($olen & 3);
     $self->{' OUTLEN'} = length($self->{' DAT'});
     $self->{' read'} = 2;           # changed from 1 to 2 so we don't read_dat() again
 # we leave numPoints and instLen since maxp stats use this
@@ -623,7 +659,7 @@ sub update_bbox
     my ($self) = @_;
     my ($num, $maxx, $minx, $maxy, $miny, $i, $comp, $x, $y, $compg);
 
-    return $self unless $self->{' read'} > 1;       # only if read_dat done
+    return $self unless (defined $self->{' read'} && $self->{' read'} > 1);       # only if read_dat done
     $miny = $minx = 65537; $maxx = $maxy = -65537;
     $num = $self->{'numberOfContours'};
     if ($num > 0)
@@ -665,6 +701,8 @@ sub update_bbox
                 $gxx += $comp->{'args'}[0];
                 $gxy += $comp->{'args'}[1];
             }
+            ($gnx, $gxx) = ($gxx, $gnx) if $gnx > $gxx;
+            ($gny, $gxy) = ($gxy, $gny) if $gny > $gxy;
             $maxx = $gxx if $gxx > $maxx;
             $minx = $gnx if $gnx < $minx;
             $maxy = $gxy if $gxy > $maxy;
@@ -775,6 +813,7 @@ sub get_points
             { ($x, $y) = ($x + $comp->{'args'}[0], $y + $comp->{'args'}[1]); }
             push (@{$self->{'x'}}, $x);
             push (@{$self->{'y'}}, $y);
+            push (@{$self->{'flags'}}, $compg->{'flags'}[$i]);
         }
         foreach $e (@{$compg->{'endPoints'}})
         { push (@{$self->{'endPoints'}}, $e + $nump); }
@@ -835,7 +874,17 @@ C<update> doesn't re-calculate the bounding box or C<numberOfContours>.
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
+
