@@ -45,11 +45,12 @@ Reads the table into data structures
 sub read
 {
     my ($self) = @_;
+    $self->SUPER::read or return $self;
+
     my ($fh) = $self->{' INFILE'};
     my ($numg, $ppem, $i, $numt, $dat, $len);
 
     $numg = $self->{' PARENT'}{'maxp'}{'numGlyphs'};
-    $self->SUPER::read or return $self;
 
     $fh->read($dat, 8);
     ($self->{'Version'}, $numt, $len) = unpack("nnN", $dat);
@@ -93,6 +94,18 @@ sub out
     $self;
 }
 
+=head2 $t->minsize()
+
+Returns the minimum size this table can be. If it is smaller than this, then the table
+must be bad and should be deleted or whatever.
+
+=cut
+
+sub minsize
+{
+    return 8;
+}
+
 
 =head2 $t->tables_do(&func)
 
@@ -111,6 +124,29 @@ sub tables_do
 }
 
 
+=head2 $t->XML_element($context, $depth, $key, $value)
+
+Outputs device metrics a little more tidily
+
+=cut
+
+sub XML_element
+{
+    my ($self) = shift;
+    my ($context, $depth, $key, $value) = @_;
+    my ($fh) = $context->{'fh'};
+    my ($i);
+
+    return $self->SUPER::XML_element(@_) if (ref($value) ne 'ARRAY');
+    $fh->print("$depth<metrics ppem='$key'>\n");
+    for ($i = 0; $i <= $#{$value}; $i += 25)
+    {
+        $fh->print("$depth$context->{'indent'}". join(' ', @{$value}[$i .. $i + 24]) . "\n");
+    }
+    $fh->print("$depth</metrics>\n");
+    $self;
+}
+
 1;
 
 =head1 BUGS
@@ -119,8 +155,18 @@ None known
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
+
