@@ -40,19 +40,37 @@ The two dates are held as an array of two unsigned longs (32-bits)
 =cut
 
 use strict;
-use vars qw(@ISA %fields);
+use vars qw(@ISA %fields @field_info);
 
 require Font::TTF::Table;
 use Font::TTF::Utils;
 
 @ISA = qw(Font::TTF::Table);
+@field_info = (
+    'version' => 'v',
+    'fontRevision' => 'f',
+    'checkSumAdjustment' => 'L',
+    'magicNumber' => 'L',
+    'flags' => 'S',
+    'unitsPerEm' => 'S',
+    'created' => 'L2',
+    'modified' => 'L2',
+    'xMin' => 's',
+    'yMin' => 's',
+    'xMax' => 's',
+    'yMax' => 's',
+    'macStyle' => 'S',
+    'lowestRecPPEM' => 'S',
+    'fontDirectionHint' => 's',
+    'indexToLocFormat' => 's',
+    'glyphDataFormat' => 's');
 
 sub init
 {
-    my ($k, $v, $c);
-    while (<Font::TTF::Head::DATA>)
+    my ($k, $v, $c, $i);
+    for ($i = 0; $i < $#field_info; $i += 2)
     {
-        ($k, $v, $c) = TTF_Init_Fields($_, $c);
+        ($k, $v, $c) = TTF_Init_Fields($field_info[$i], $c, $field_info[$i + 1]);
         next unless defined $k && $k ne "";
         $fields{$k} = $v;
     }
@@ -99,6 +117,19 @@ sub out
 }
 
 
+=head2 $t->minsize()
+
+Returns the minimum size this table can be. If it is smaller than this, then the table
+must be bad and should be deleted or whatever.
+
+=cut
+
+sub minsize
+{
+    return 54;
+}
+
+
 =head2 $t->XML_element($context, $depth, $key, $value)
 
 Handles date process for the XML exporter
@@ -142,29 +173,27 @@ sub update
     $hmtx = $self->{' PARENT'}{'hmtx'}->read;
     
     $self->{' PARENT'}{'loca'}->update;
-    unless ($hmtx->update)              # if we updated, then the flags will be set anyway.
+    $hmtx->update;              # if we updated, then the flags will be set anyway.
+    $lsbx = 1;
+    for ($i = 0; $i < $num; $i++)
     {
-        $lsbx = 1;
-        for ($i = 0; $i < $num; $i++)
-        {
-            $loc = $self->{' PARENT'}{'loca'}{'glyphs'}[$i];
-            next unless defined $loc;
-            $loc->read;
-            $xMin = $loc->{'xMin'} if ($loc->{'xMin'} < $xMin || $i == 0);
-            $yMin = $loc->{'yMin'} if ($loc->{'yMin'} < $yMin || $i == 0);
-            $xMax = $loc->{'xMax'} if ($loc->{'xMax'} > $xMax);
-            $yMax = $loc->{'yMax'} if ($loc->{'yMax'} > $yMax);
-            $lsbx &= ($loc->{'xMin'} == $hmtx->{'lsb'}[$i]);
-        }
-        $self->{'xMin'} = $xMin;
-        $self->{'yMin'} = $yMin;
-        $self->{'xMax'} = $xMax;
-        $self->{'yMax'} = $yMax;
-        if ($lsbx)
-        { $self->{'flags'} |= 2; }
-        else
-        { $self->{'flags'} &= ~2; }
+        $loc = $self->{' PARENT'}{'loca'}{'glyphs'}[$i];
+        next unless defined $loc;
+        $loc->read->update_bbox;
+        $xMin = $loc->{'xMin'} if ($loc->{'xMin'} < $xMin || $i == 0);
+        $yMin = $loc->{'yMin'} if ($loc->{'yMin'} < $yMin || $i == 0);
+        $xMax = $loc->{'xMax'} if ($loc->{'xMax'} > $xMax);
+        $yMax = $loc->{'yMax'} if ($loc->{'yMax'} > $yMax);
+        $lsbx &= ($loc->{'xMin'} == $hmtx->{'lsb'}[$i]);
     }
+    $self->{'xMin'} = $xMin;
+    $self->{'yMin'} = $yMin;
+    $self->{'xMax'} = $xMax;
+    $self->{'yMax'} = $yMax;
+    if ($lsbx)
+    { $self->{'flags'} |= 2; }
+    else
+    { $self->{'flags'} &= ~2; }
     $self;
 }
 
@@ -180,16 +209,16 @@ storable time.
 sub getdate
 {
     my ($self, $is_create) = @_;
-    my ($arr) = $self->{$is_create ? 'created' : 'modified'};
+    my (@arr) = (@{$self->{$is_create ? 'created' : 'modified'}});
 
-    $arr->[1] -= 2082844800;        # seconds between 1/Jan/1904 and 1/Jan/1970 (midnight)
-    if ($arr->[1] < 0)
+    $arr[1] -= 2082844800;        # seconds between 1/Jan/1904 and 1/Jan/1970 (midnight)
+    if ($arr[1] < 0)
     {
-        $arr->[1] += 0xFFFFFFF; $arr->[1]++;
-        $arr->[0]--;
+        $arr[1] += 0xFFFFFFF; $arr[1]++;
+        $arr[0]--;
     }
-    return undef if $arr->[0] != 0;
-    return $arr->[1];
+    return undef if $arr[0] != 0;
+    return $arr[1];
 }
 
 
@@ -227,28 +256,18 @@ None known
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
 
-__DATA__
-version, f
-fontRevision, f
-checkSumAdjustment, L
-magicNumber, L
-flags, S
-unitsPerEm, S
-created, L2
-modified, L2
-xMin, s
-yMin, s
-xMax, s
-yMax, s
-macStyle, S
-lowestRecPPEM, S
-fontDirectionHint, s
-indexToLocFormat, s
-glyphDataFormat, s
-
