@@ -2,7 +2,7 @@ package Font::TTF::Hhea;
 
 =head1 NAME
 
-TTF:Hhea - Horizontal Header table
+Font::TTF::Hhea - Horizontal Header table
 
 =head1 DESCRIPTION
 
@@ -36,7 +36,7 @@ use Font::TTF::Utils;
 
 @ISA = qw(Font::TTF::Table);
 @field_info = (
-    'version' => 'f',
+    'version' => 'v',
     'Ascender' => 's',
     'Descender' => 's',
     'LineGap' => 's',
@@ -99,10 +99,22 @@ sub out
 }
 
 
+=head2 $t->minsize()
+
+Returns the minimum size this table can be. If it is smaller than this, then the table
+must be bad and should be deleted or whatever.
+
+=cut
+
+sub minsize
+{
+    return 36;
+}
+
+
 =head2 $t->update
 
-Updates various parameters in the hhea table from the hmtx table, assuming
-the C<hmtx> table is dirty.
+Updates various parameters in the hhea table from the hmtx table.
 
 =cut
 
@@ -116,14 +128,12 @@ sub update
 
     return undef unless ($self->SUPER::update);
     return undef unless (defined $hmtx && defined $self->{' PARENT'}{'loca'});
-    
-    $res = $hmtx->read->update;
-    $res |= $self->{' PARENT'}{'loca'}->read->update;
+
+    $hmtx->read->update;
+    $self->{' PARENT'}{'loca'}->read->update;
     $glyphs = $self->{' PARENT'}{'loca'}{'glyphs'};
     $num = $self->{' PARENT'}{'maxp'}{'numGlyphs'};
 
-    return undef unless ($res);
-    
     for ($i = 0; $i < $num; $i++)
     {
         $aw = $hmtx->{'advance'}[$i];
@@ -155,8 +165,18 @@ None known
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
+
