@@ -51,17 +51,17 @@ sub read
     my ($numh, $numg);
 
     $numh = $self->{' PARENT'}{'hhea'}->read->{'numberOfHMetrics'};
-    $numg = $self->{' PARENT'}{'maxp'}->read->{'numGlyphs'};
+    $numg = $self->{' PARENT'}{'maxp'}{'numGlyphs'};
     $self->_read($numg, $numh, "advance", "lsb");
 }
 
 sub _read
 {
     my ($self, $numg, $numh, $tAdv, $tLsb) = @_;
+    $self->SUPER::read or return $self;
+
     my ($fh) = $self->{' INFILE'};
     my ($i, $dat);
-    
-    $self->SUPER::read or return $self;
 
     for ($i = 0; $i < $numh; $i++)
     {
@@ -71,7 +71,7 @@ sub _read
     }
     
     $i--;
-    while ($i++ < $numg)
+    while (++$i < $numg)
     {
         $fh->read($dat, 2);
         $self->{$tAdv}[$i] = $self->{$tAdv}[$numh - 1];
@@ -138,6 +138,67 @@ sub _out
 }
 
 
+=head2 $t->update
+
+Updates the lsb values from the xMin from the each glyph
+
+=cut
+
+sub update
+{
+    my ($self) = @_;
+    my ($numg) = $self->{' PARENT'}{'maxp'}{'numGlyphs'};
+    my ($i);
+
+    return undef unless ($self->SUPER::update);
+# lsb & xMin must always be the same, regardless of any flags!
+#    return $self unless ($self->{' PARENT'}{'head'}{'flags'} & 2);        # lsb & xMin the same
+
+    $self->{' PARENT'}{'loca'}->update;
+    for ($i = 0; $i < $numg; $i++)
+    {
+        my ($g) = $self->{' PARENT'}{'loca'}{'glyphs'}[$i];
+        if ($g)
+        { $self->{'lsb'}[$i] = $g->read->update_bbox->{'xMin'}; }
+        else
+        { $self->{'lsb'}[$i] = 0; }
+    }
+    $self->{' PARENT'}{'head'}{'flags'} |= 2;
+    $self;
+}
+    
+
+=head2 $t->out_xml($context, $depth)
+
+Outputs the table in XML
+
+=cut
+
+sub out_xml
+{
+    my ($self, $context, $depth) = @_;
+    my ($fh) = $context->{'fh'};
+    my ($numg) = $self->{' PARENT'}{'maxp'}{'numGlyphs'};
+    my ($addr) = ($self =~ m/\((.+)\)$/o);
+    my ($i);
+
+    if ($context->{'addresses'}{$addr})
+    {
+        $fh->printf("%s<%s id_ref='%s'/>\n", $depth, $context->{'name'}, $addr);
+        return $self;
+    }
+    else
+    { $fh->printf("%s<%s id='%s'>\n", $depth, $context->{'name'}, $addr); }
+
+    $self->read;
+
+    for ($i = 0; $i < $numg; $i++)
+    { $fh->print("$depth$context->{'indent'}<width gid='$i' adv='$self->{'advance'}[$i]' lsb='$self->{'lsb'}[$i]'/>\n"); }
+
+    $fh->print("$depth</$context->{'name'}>\n");
+    $self;
+}
+
 1;
 
 =head1 BUGS
@@ -146,8 +207,18 @@ None known
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
+
