@@ -2,7 +2,7 @@ package Font::TTF::Kern::CompactClassArray;
 
 =head1 NAME
 
-Font::TTF::Kern::CompactClassArray
+Font::TTF::Kern::CompactClassArray - Compact Class Array kern subtable for AAT
 
 =head1 METHODS
 
@@ -86,8 +86,18 @@ None known
 
 =head1 AUTHOR
 
-Jonathan Kew L<Jonathan_Kew@sil.org>. See L<Font::TTF::Font> for copyright and
-licensing.
+Jonathan Kew L<Jonathan_Kew@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
+
