@@ -10,7 +10,7 @@ The location table holds the directory of locations of each glyph within the
 glyf table. Due to this relationship and the unimportance of the actual locations
 when it comes to holding glyphs in memory, reading the location table results
 in the creation of glyph objects for each glyph and stores them here.
-So if you are looking for glyphs, don't look in the C<glyf> table, look here
+So if you are looking for glyphs, do not look in the C<glyf> table, look here
 instead.
 
 Things get complicated if you try to change the glyph list within the one table.
@@ -46,6 +46,20 @@ use vars qw(@ISA);
 require Font::TTF::Glyph;
 
 
+=head2 $t->new
+
+Creates a new location table making sure it has a glyphs array
+
+=cut
+
+sub new
+{
+    my ($class) = shift;
+    my ($res) = $class->SUPER::new(@_);
+    $res->{'glyphs'} = [];
+    $res;
+}
+
 =head2 $t->read
 
 Reads the location table creating glyph objects (L<Font::TTF::Glyph>) for each glyph
@@ -56,13 +70,17 @@ allowing their later reading.
 sub read
 {
     my ($self) = @_;
-    my ($fh) = $self->{' INFILE'};
-    my ($locFmt) = $self->{' PARENT'}{'head'}->read->{'indexToLocFormat'};
-    my ($numGlyphs) = $self->{' PARENT'}{'maxp'}->read->{'numGlyphs'};
-    my ($glyfLoc) = $self->{' PARENT'}{'glyf'}{' OFFSET'};
-    my ($dat, $last, $i, $loc);
+
+	# Do this before $self->SUPER::read because this can alter the file pointer:
+    my ($glyfLoc) = $self->{' PARENT'}{'glyf'}->_read->{' OFFSET'};		# May seek on $fh!
 
     $self->SUPER::read or return $self;
+
+    my ($fh) = $self->{' INFILE'};
+    my ($locFmt) = $self->{' PARENT'}{'head'}{'indexToLocFormat'};
+    my ($numGlyphs) = $self->{' PARENT'}{'maxp'}{'numGlyphs'};
+    my ($dat, $last, $i, $loc);
+
     $fh->read($dat, $locFmt ? 4 : 2);
     $last = unpack($locFmt ? "N" : "n", $dat);
     for ($i = 0; $i < $numGlyphs; $i++)
@@ -72,8 +90,8 @@ sub read
         $self->{'glyphs'}[$i] = ($self->{'glyphtype'} || "Font::TTF::Glyph")->new(
                 LOC => $last << ($locFmt ? 0 : 1),
                 OUTLOC => $last << ($locFmt ? 0 : 1),
-                ' PARENT' => $self->{' PARENT'},
-                INFILE => $fh,
+                PARENT => $self->{' PARENT'},
+                INFILE => $self->{' PARENT'}{'glyf'}{' INFILE'},
                 BASE => $glyfLoc,
                 OUTLEN => ($loc - $last) << ($locFmt ? 0 : 1),
                 LEN => ($loc - $last) << ($locFmt ? 0 : 1)) if ($loc != $last);
@@ -98,8 +116,8 @@ table was attempted to be output.
 sub out
 {
     my ($self, $fh) = @_;
-    my ($locFmt) = $self->{' PARENT'}{'head'}->read->{'indexToLocFormat'};
-    my ($numGlyphs) = $self->{' PARENT'}{'maxp'}->read->{'numGlyphs'};
+    my ($locFmt) = $self->{' PARENT'}{'head'}{'indexToLocFormat'};
+    my ($numGlyphs) = $self->{' PARENT'}{'maxp'}{'numGlyphs'};
     my ($count, $i, $offset, $g);
 
     return $self->SUPER::out($fh) unless ($self->{' read'});
@@ -115,17 +133,27 @@ sub out
         } else
         {
             if ($locFmt)
-            { $fh->print(pack("N", $g->{'OUTLOC'}) x ($count + 1)); }
+            { $fh->print(pack("N", $g->{' OUTLOC'}) x ($count + 1)); }
             else
-            { $fh->print(pack("n", $g->{'OUTLOC'} >> 1) x ($count + 1)); }
+            { $fh->print(pack("n", $g->{' OUTLOC'} >> 1) x ($count + 1)); }
             $count = 0;
-            $offset = $g->{'OUTLOC'} + $g->{'OUTLEN'};
+            $offset = $g->{' OUTLOC'} + $g->{' OUTLEN'};
         }
     }
     $fh->print(pack($locFmt ? "N" : "n", ($locFmt ? $offset: $offset >> 1)) x ($count + 1));
 }
 
 
+=head2 $t->out_xml($context, $depth)
+
+No need to output a loca table, this is dynamically generated
+
+=cut
+
+sub out_xml
+{ return $_[0]; }
+
+
 =head2 $t->glyphs_do(&func)
 
 Calls func for each glyph in this location table in numerical order:
@@ -152,8 +180,18 @@ None known
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
+
