@@ -25,6 +25,12 @@ Flag which indicates that the table has already been read from file.
 Allows the creation of unspecific tables. Data is simply output to any font
 file being created.
 
+=item nocompress
+
+If set, overrides the font default for WOFF table compression. Is a scalar integer specifying a 
+table size threshold below which this table will not be compressed. Set to -1 to never
+compress; 0 to always compress.
+
 =item INFILE
 
 The read file handle
@@ -37,6 +43,10 @@ Location of the file in the input file
 
 Length in the input directory
 
+=item ZLENGTH
+
+Compressed length of the table if a WOFF font. 0 < ZLENGTH < LENGTH implies table is compressed.
+
 =item CSUM
 
 Checksum read from the input file's directory
@@ -54,9 +64,11 @@ The L<Font::TTF::Font> that table is part of
 use strict;
 use vars qw($VERSION);
 use Font::TTF::Utils;
-
+use IO::String;
 $VERSION = 0.0001;
 
+my $havezlib = eval {require Compress::Zlib};
+	
 =head2 Font::TTF::Table->new(%parms)
 
 Creates a new table or subclass. Table instance variables are passed in
@@ -93,6 +105,18 @@ sub read
     return $self->read_dat if (ref($self) eq "Font::TTF::Table");
     return undef if $self->{' read'};
     $self->{' INFILE'}->seek($self->{' OFFSET'}, 0);
+    if (0 < $self->{' ZLENGTH'} && $self->{' ZLENGTH'} < $self->{' LENGTH'})
+    {
+    	# WOFF table is compressed. Uncompress it to memory and create new fh
+    	die ("Cannot uncompress WOFF data: Compress::Zlib not present.\n") unless $havezlib;
+    	my $dat;
+    	$self->{' INFILE'}->read($dat, $self->{' ZLENGTH'}); 
+    	$dat = Compress::Zlib::uncompress($dat);
+    	warn "$self->{' NAME'} table decompressed to wrong length" if $self->{' LENGTH'} != bytes::length($dat);
+        $self->{' INFILE'} = IO::String->new($dat);
+        binmode $self->{' INFILE'};
+        $self->{' OFFSET'} = 0;
+    }
     $self->{' read'} = 1;
     $self;
 }
@@ -114,14 +138,27 @@ sub read_dat
     return undef if ($self->{' read'});
 #    $self->{' read'} = 1;      # Let read do this, now out will call us for subclasses
     $self->{' INFILE'}->seek($self->{' OFFSET'}, 0);
-    $self->{' INFILE'}->read($self->{' dat'}, $self->{' LENGTH'});
+    if (0 < $self->{' ZLENGTH'} && $self->{' ZLENGTH'} < $self->{' LENGTH'})
+    {
+    	# WOFF table is compressed. Uncompress it directly to ' dat'
+    	die ("Cannot uncompress WOFF data: Compress::Zlib not present.\n") unless $havezlib;
+    	my $dat;
+    	$self->{' INFILE'}->read($dat, $self->{' ZLENGTH'}); 
+    	$dat = Compress::Zlib::uncompress($dat);
+    	warn "$self->{' NAME'} table decompressed to wrong length" if $self->{' LENGTH'} != bytes::length($dat);
+    	$self->{' dat'} = $dat;
+ 	}
+ 	else
+ 	{
+	    $self->{' INFILE'}->read($self->{' dat'}, $self->{' LENGTH'});
+	}
     $self;
 }
 
 =head2 $t->out($fh)
 
 Writes out the table to the font file. If there is anything in the
-C<data> instance variable then this is output, otherwise the data is copied
+C<dat> instance variable then this is output, otherwise the data is copied
 from the input file to the output
 
 =cut
@@ -188,7 +225,7 @@ Output a particular element based on its contents.
 
 sub XML_element
 {
-    my ($self, $context, $depth, $k, $dat) = @_;
+    my ($self, $context, $depth, $k, $dat, $ind) = @_;
     my ($fh) = $context->{'fh'};
     my ($ndepth, $d);
 
@@ -200,15 +237,19 @@ sub XML_element
         return $self;
     }
 
-    $fh->printf("%s<%s>\n", $depth, $k);
+    if ($ind)
+    { $fh->printf("%s<%s i='%d'>\n", $depth, $k, $ind); }
+    else
+    { $fh->printf("%s<%s>\n", $depth, $k); }
     $ndepth = $depth . $context->{'indent'};
 
     if (ref($dat) eq 'SCALAR')
     { $self->XML_element($context, $ndepth, 'scalar', $$dat); }
     elsif (ref($dat) eq 'ARRAY')
     {
+        my ($c) = 1;
         foreach $d (@{$dat})
-        { $self->XML_element($context, $ndepth, 'elem', $d); }
+        { $self->XML_element($context, $ndepth, 'elem', $d, $c++); }
     }
     elsif (ref($dat) eq 'HASH')
     {
@@ -250,6 +291,18 @@ sub XML_end
 }
     
 
+=head2 $t->minsize()
+
+Returns the minimum size this table can be. If it is smaller than this, then the table
+must be bad and should be deleted or whatever.
+
+=cut
+
+sub minsize
+{
+    return 0;
+}
+
 =head2 $t->dirty($val)
 
 This sets the dirty flag to the given value or 1 if no given value. It returns the
@@ -317,7 +370,7 @@ sub empty
 
 Releases ALL of the memory used by this table, and all of its component/child
 objects.  This method is called automatically by
-'C<Font::TTF::Font-E<GT>release>' (so you don't have to call it yourself).
+'Font::TTF::Font-E<gt>release' (so you don't have to call it yourself).
 
 B<NOTE>, that it is important that this method get called at some point prior
 to the actual destruction of the object.  Internally, we track things in a
@@ -375,8 +428,18 @@ No known bugs
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
+
