@@ -110,7 +110,7 @@ table of this particular feature.
 
 =item PARMS
 
-This is an unused offset to the parameters for each feature
+If FeatureParams are defined for this feature, this contains a reference to the corresponding FeatureParams object.  Otherwise set to null.
 
 =item LOOKUPS
 
@@ -144,6 +144,10 @@ within the lookup
 
 Holds the lookup flag bits
 
+=item FILTER
+
+Holds the MarkFilteringSet (that is, the index into GDEF->MARKSETS) for the lookup.
+
 =item SUB
 
 This holds an array of subtables which are subclass specific. Each subtable must have
@@ -222,10 +226,6 @@ The index to a lookup to be acted upon on the match string.
 
 =back
 
-=back
-
-=back
-
 =item CLASS
 
 For those lookups which use class categories rather than glyph ids for matching
@@ -240,6 +240,8 @@ This is the offset to the class definition for the before match glyphs
 
 This is the offset to the class definition for the after match glyphs.
 
+=back
+
 =item ACTION_TYPE
 
 This string holds the type of information held in the ACTION variable of a RULE.
@@ -268,6 +270,7 @@ are stored relative to another base within the subtable.
 
 =back
 
+=back
 
 =head1 METHODS
 
@@ -276,11 +279,18 @@ are stored relative to another base within the subtable.
 use Font::TTF::Table;
 use Font::TTF::Utils;
 use Font::TTF::Coverage;
+
 use strict;
-use vars qw(@ISA);
+use vars qw(@ISA %FeatParams);
 
 @ISA = qw(Font::TTF::Table);
 
+%FeatParams = (
+    'ss' => 'Font::TTF::Features::Sset',
+  'cv' => 'Font::TTF::Features::Cvar',
+  'si' => 'Font::TTF::Features::Size',
+  );
+
 =head2 $t->read
 
 Reads the table passing control to the subclass to handle the subtable specifics
@@ -290,14 +300,15 @@ Reads the table passing control to the subclass to handle the subtable specifics
 sub read
 {
     my ($self) = @_;
+    $self->SUPER::read or return $self;
+
     my ($dat, $i, $l, $oScript, $oFeat, $oLook, $tag, $nScript, $off, $dLang, $nLang, $lTag);
-    my ($nFeat, $nLook, $nSub, $j, $temp);
+    my ($nFeat, $oParms, $FType, $nLook, $nSub, $j, $temp, $t);
     my ($fh) = $self->{' INFILE'};
     my ($moff) = $self->{' OFFSET'};
 
-    $self->SUPER::read or return $self;
     $fh->read($dat, 10);
-    ($self->{'Version'}, $oScript, $oFeat, $oLook) = TTF_Unpack("fSSS", $dat);
+    ($self->{'Version'}, $oScript, $oFeat, $oLook) = TTF_Unpack("vSSS", $dat);
 
 # read features first so that in the script/lang hierarchy we can use feature tags
 
@@ -309,26 +320,45 @@ sub read
     $fh->read($dat, 6 * $nFeat);
     for ($i = 0; $i < $nFeat; $i++)
     {
-    	($tag, $off) = unpack("a4n", substr($dat, $i * 6, 6));
-    	while (defined $l->{$tag})
-    	{
-    	    if ($tag =~ m/(.*?)\s_(\d+)$/o)
-    	    { $tag = $1 . " _" . ($2 + 1); }
-    	    else
-    	    { $tag .= " _0"; }
-    	}
-	    $l->{$tag}{' OFFSET'} = $off + $oFeat;
-	    $l->{$tag}{'INDEX'} = $i;
-	    push (@{$l->{'FEAT_TAGS'}}, $tag);
+        ($tag, $off) = unpack("a4n", substr($dat, $i * 6, 6));
+        while (defined $l->{$tag})
+        {
+            if ($tag =~ m/(.*?)\s_(\d+)$/o)
+            { $tag = $1 . " _" . ($2 + 1); }
+            else
+            { $tag .= " _0"; }
+        }
+        $l->{$tag}{' OFFSET'} = $off + $oFeat;
+        $l->{$tag}{'INDEX'} = $i;
+        push (@{$l->{'FEAT_TAGS'}}, $tag);
     }
 
     foreach $tag (grep {m/^.{4}(?:\s_\d+)?$/o} keys %$l)
     {
-	    $fh->seek($moff + $l->{$tag}{' OFFSET'}, 0);
-    	$fh->read($dat, 4);
-	    ($l->{$tag}{'PARMS'}, $nLook) = unpack("n2", $dat);
-    	$fh->read($dat, $nLook * 2);
-	    $l->{$tag}{'LOOKUPS'} = [unpack("n*", $dat)];
+        $oFeat=$moff + $l->{$tag}{' OFFSET'};
+        $fh->seek($oFeat, 0);
+        $fh->read($dat, 4);
+        ($oParms, $nLook) = unpack("n2", $dat);
+        $fh->read($dat, $nLook * 2);
+        $l->{$tag}{'LOOKUPS'} = [unpack("n*", $dat)];
+        $l->{$tag}{'PARMS'}="";
+        if ($oParms > 0)
+        {
+            $FType=$FeatParams{substr($tag,0,2)};
+            if ($FType)
+            {
+                $t=$FType;
+                if ($^O eq "MacOS")
+                    { $t =~ s/^|::/:/oig; }
+                else
+                    { $t =~ s|::|/|oig; }
+                    require "$t.pm";
+                $l->{$tag}{'PARMS'} = $FType->new( INFILE  => $fh,
+                                                                                     OFFSET => $oFeat+$oParms);
+            $l->{$tag}{'PARMS'}->read;
+          }
+        }               
+        
     }
 
 # Now the script/lang hierarchy
@@ -341,51 +371,52 @@ sub read
     $fh->read($dat, 6 * $nScript);
     for ($i = 0; $i < $nScript; $i++)
     {
-    	($tag, $off) = unpack("a4n", substr($dat, $i * 6, 6));
-    	$off += $oScript;
+        ($tag, $off) = unpack("a4n", substr($dat, $i * 6, 6));
+        $off += $oScript;
         foreach (keys %$l)
         { $l->{$tag}{' REFTAG'} = $_ if ($l->{$_}{' OFFSET'} == $off
                                         && !defined $l->{$_}{' REFTAG'}); }
-	    $l->{$tag}{' OFFSET'} = $off;
+        $l->{$tag}{' OFFSET'} = $off;
     }
 
     foreach $tag (keys %$l)
     {
         next if ($l->{$tag}{' REFTAG'});
-    	$fh->seek($moff + $l->{$tag}{' OFFSET'}, 0);
-    	$fh->read($dat, 4);
-    	($dLang, $nLang) = unpack("n2", $dat);
-    	$l->{$tag}{'DEFAULT'}{' OFFSET'} =
-    	        $dLang + $l->{$tag}{' OFFSET'} if $dLang;
-    	$fh->read($dat, 6 * $nLang);
-    	for ($i = 0; $i < $nLang; $i++)
-    	{
-    	    ($lTag, $off) = unpack("a4n", substr($dat, $i * 6, 6));
-    	    $off += $l->{$tag}{' OFFSET'};
-    	    $l->{$tag}{$lTag}{' OFFSET'} = $off;
+        $fh->seek($moff + $l->{$tag}{' OFFSET'}, 0);
+        $fh->read($dat, 4);
+        ($dLang, $nLang) = unpack("n2", $dat);
+        $l->{$tag}{'DEFAULT'}{' OFFSET'} =
+                $dLang + $l->{$tag}{' OFFSET'} if $dLang;
+        $fh->read($dat, 6 * $nLang);
+        for ($i = 0; $i < $nLang; $i++)
+        {
+            ($lTag, $off) = unpack("a4n", substr($dat, $i * 6, 6));
+            $off += $l->{$tag}{' OFFSET'};
+            $l->{$tag}{$lTag}{' OFFSET'} = $off;
             foreach (@{$l->{$tag}{'LANG_TAGS'}}, 'DEFAULT')
             { $l->{$tag}{$lTag}{' REFTAG'} = $_ if ($l->{$tag}{$_}{' OFFSET'} == $off
                                                    && !$l->{$tag}{$_}{' REFTAG'}); }
-    	    push (@{$l->{$tag}{'LANG_TAGS'}}, $lTag);
-    	}
-    	foreach $lTag (@{$l->{$tag}{'LANG_TAGS'}}, 'DEFAULT')
-    	{
-    	    next unless defined $l->{$tag}{$lTag};
+            push (@{$l->{$tag}{'LANG_TAGS'}}, $lTag);
+        }
+        foreach $lTag (@{$l->{$tag}{'LANG_TAGS'}}, 'DEFAULT')
+        {
+            next unless defined $l->{$tag}{$lTag};
             next if ($l->{$tag}{$lTag}{' REFTAG'});
-    	    $fh->seek($moff + $l->{$tag}{$lTag}{' OFFSET'}, 0);
-    	    $fh->read($dat, 6);
-    	    ($l->{$tag}{$lTag}{'RE-ORDER'}, $l->{$tag}{$lTag}{'DEFAULT'}, $nFeat) 
-    	      = unpack("n3", $dat);
-    	    $fh->read($dat, $nFeat * 2);
-    	    $l->{$tag}{$lTag}{'FEATURES'} = [map {$self->{'FEATURES'}{'FEAT_TAGS'}[$_]} unpack("n*", $dat)];
-    	}
-    	foreach $lTag (@{$l->{$tag}{'LANG_TAGS'}}, 'DEFAULT')
-    	{
-       	    next unless $l->{$tag}{$lTag}{' REFTAG'};
-    	    $temp = $l->{$tag}{$lTag}{' REFTAG'};
-    	    $l->{$tag}{$lTag} = &copy($l->{$tag}{$temp});
-    	    $l->{$tag}{$lTag}{' REFTAG'} = $temp;
-    	}
+            $fh->seek($moff + $l->{$tag}{$lTag}{' OFFSET'}, 0);
+            $fh->read($dat, 6);
+            ($l->{$tag}{$lTag}{'RE-ORDER'}, $l->{$tag}{$lTag}{'DEFAULT'}, $nFeat) 
+              = unpack("n3", $dat);
+            $fh->read($dat, $nFeat * 2);
+            $l->{$tag}{$lTag}{'FEATURES'} = [map {$self->{'FEATURES'}{'FEAT_TAGS'}[$_]} unpack("n*", $dat)];
+        }
+        foreach $lTag (@{$l->{$tag}{'LANG_TAGS'}}, 'DEFAULT')
+        {
+        	# Make copies of referenced languages for each reference. 
+            next unless $l->{$tag}{$lTag}{' REFTAG'};
+            $temp = $l->{$tag}{$lTag}{' REFTAG'};
+            $l->{$tag}{$lTag} = &copy($l->{$tag}{$temp});
+            $l->{$tag}{$lTag}{' REFTAG'} = $temp;
+        }
     }
     foreach $tag (keys %$l)
     {
@@ -406,28 +437,32 @@ sub read
 
     for ($i = 0; $i < $nLook; $i++)
     {
-    	$l = $self->{'LOOKUP'}[$i];
-    	$fh->seek($l->{' OFFSET'} + $moff + $oLook, 0);
-    	$fh->read($dat, 6);
-    	($l->{'TYPE'}, $l->{'FLAG'}, $nSub) = unpack("n3", $dat);
-    	$fh->read($dat, $nSub * 2);
-    	$j = 0;
+        $l = $self->{'LOOKUP'}[$i];
+        $fh->seek($l->{' OFFSET'} + $moff + $oLook, 0);
+        $fh->read($dat, 6);
+        ($l->{'TYPE'}, $l->{'FLAG'}, $nSub) = unpack("n3", $dat);
+        $fh->read($dat, $nSub * 2);
         my @offsets = unpack("n*", $dat);
+        if ($l->{'FLAG'} & 0x0010)
+        {
+            $fh->read($dat, 2);
+            $l->{'FILTER'} = unpack("n", $dat);
+        }
         my $isExtension = ($l->{'TYPE'} == $self->extension());
-    	for ($j = 0; $j < $nSub; $j++)
-    	{
-    	    $l->{'SUB'}[$j]{' OFFSET'} = $offsets[$j];
-       	    $fh->seek($moff + $oLook + $l->{' OFFSET'} + $l->{'SUB'}[$j]{' OFFSET'}, 0);
-    	    if ($isExtension)
-    	    {
-    	        $fh->read($dat, 8);
-    	        my $longOff;
-    	        (undef, $l->{'TYPE'}, $longOff) = unpack("nnN", $dat);
-    	        $l->{'SUB'}[$j]{' OFFSET'} += $longOff;
-        	    $fh->seek($moff + $oLook + $l->{' OFFSET'} + $l->{'SUB'}[$j]{' OFFSET'}, 0);
+        for ($j = 0; $j < $nSub; $j++)
+        {
+            $l->{'SUB'}[$j]{' OFFSET'} = $offsets[$j];
+            $fh->seek($moff + $oLook + $l->{' OFFSET'} + $l->{'SUB'}[$j]{' OFFSET'}, 0);
+            if ($isExtension)
+            {
+                $fh->read($dat, 8);
+                my $longOff;
+                (undef, $l->{'TYPE'}, $longOff) = unpack("nnN", $dat);
+                $l->{'SUB'}[$j]{' OFFSET'} += $longOff;
+                $fh->seek($moff + $oLook + $l->{' OFFSET'} + $l->{'SUB'}[$j]{' OFFSET'}, 0);
             }
-	        $self->read_sub($fh, $l, $j);
-	    }
+            $self->read_sub($fh, $l, $j);
+        }
     }
     return $self;
 }
@@ -459,7 +494,7 @@ sub extension
 Writes this Opentype table to the output calling $t->out_sub for each sub table
 at the appropriate point in the output. The assumption is that on entry the
 number of scripts, languages, features, lookups, etc. are all resolved and
-the relationships fixed. This includes a script's LANG_TAGS list and that all
+the relationships fixed. This includes a LANG_TAGS list for a script, and that all
 scripts and languages in their respective dictionaries either have a REFTAG or contain
 real data.
 
@@ -469,8 +504,8 @@ sub out
 {
     my ($self, $fh) = @_;
     my ($i, $j, $base, $off, $tag, $t, $l, $lTag, $oScript, @script, @tags);
-    my ($end, $nTags, @offs, $oFeat, $oLook, $nSub, $nSubs, $big, $out);
-
+    my ($end, $nTags, @offs, $oFeat, $oFtable, $oParms, $FType, $oLook, $nSub, $nSubs, $big, $out);
+    
     return $self->SUPER::out($fh) unless $self->{' read'};
 
 # First sort the features
@@ -481,7 +516,7 @@ sub out
     { $self->{'FEATURES'}{$t}{'INDEX'} = $i++; }
 
     $base = $fh->tell();
-    $fh->print(TTF_Pack("f", $self->{'Version'}));
+    $fh->print(TTF_Pack("v", $self->{'Version'}));
     $fh->print(pack("n3", 10, 0, 0));
     $oScript = $fh->tell() - $base;
     @script = sort grep {length($_) == 4} keys %{$self->{'SCRIPTS'}};
@@ -495,40 +530,53 @@ sub out
         $fh->seek($end, 0);
         $tag = $self->{'SCRIPTS'}{$t};
         next if ($tag->{' REFTAG'});
-    	$tag->{' OFFSET'} = tell($fh) - $base - $oScript;
-    	$fh->print(pack("n2", 0, $#{$tag->{'LANG_TAGS'}} + 1));
-    	foreach $lTag (sort @{$tag->{'LANG_TAGS'}})
-    	{ $fh->print(pack("a4n", $lTag, 0)); }
-    	foreach $lTag (@{$tag->{'LANG_TAGS'}}, 'DEFAULT')
-    	{
-    	    my ($def);
-    	    $l = $tag->{$lTag};
-    	    next if (!defined $l || (defined $l->{' REFTAG'} && $l->{' REFTAG'} ne ''));
-    	    $l->{' OFFSET'} = tell($fh) - $base - $oScript - $tag->{' OFFSET'};
-    	    if (defined $l->{'DEFAULT'})
-#    	    { $def = $self->{'FEATURES'}{$l->{'FEATURES'}[$l->{'DEFAULT'}]}{'INDEX'}; }
+        $tag->{' OFFSET'} = tell($fh) - $base - $oScript;
+        $fh->print(pack("n2", 0, $#{$tag->{'LANG_TAGS'}} + 1));
+        foreach $lTag (sort @{$tag->{'LANG_TAGS'}})
+        { $fh->print(pack("a4n", $lTag, 0)); }
+        foreach $lTag (@{$tag->{'LANG_TAGS'}}, 'DEFAULT')
+        {
+            my ($def);
+            $l = $tag->{$lTag};
+            next if (!defined $l || (defined $l->{' REFTAG'} && $l->{' REFTAG'} ne ''));
+            $l->{' OFFSET'} = $fh->tell() - $base - $oScript - $tag->{' OFFSET'};
+            if (defined $l->{'DEFAULT'})
+#           { $def = $self->{'FEATURES'}{$l->{'FEATURES'}[$l->{'DEFAULT'}]}{'INDEX'}; }
             { $def = $l->{'DEFAULT'}; }
-    	    else
-    	    { $def = -1; }
-    	    $fh->print(pack("n*", $l->{'RE_ORDER'} || 0, $def, $#{$l->{'FEATURES'}} + 1,
-    	            map {$self->{'FEATURES'}{$_}{'INDEX'} || 0} @{$l->{'FEATURES'}}));
-    	}
-    	$end = $fh->tell();
-    	if ($tag->{'DEFAULT'}{' REFTAG'} || defined $tag->{'DEFAULT'}{'FEATURES'})
-    	{
-        	$fh->seek($base + $oScript + $tag->{' OFFSET'}, 0);
-        	$off = $tag->{'DEFAULT'}{' REFTAG'} ?
-        	        $tag->{$tag->{'DEFAULT'}{' REFTAG'}}{' OFFSET'} :
-        	        $tag->{'DEFAULT'}{' OFFSET'};
-        	$fh->print(pack("n", $off));
-    	}
-    	$fh->seek($base + $oScript + $tag->{' OFFSET'} + 4, 0);
-    	foreach (sort @{$tag->{'LANG_TAGS'}})
-    	{
-    	    $off = $tag->{$_}{' REFTAG'} ? $tag->{$tag->{$_}{' REFTAG'}}{' OFFSET'} :
-    	            $tag->{$_}{' OFFSET'};
-    	    $fh->print(pack("a4n", $_, $off));
-    	}
+            else
+            { $def = -1; }
+            $fh->print(pack("n*", $l->{'RE_ORDER'} || 0, $def, $#{$l->{'FEATURES'}} + 1,
+                    map {$self->{'FEATURES'}{$_}{'INDEX'} || 0} @{$l->{'FEATURES'}}));
+        }
+        $end = $fh->tell();
+        if ($tag->{'DEFAULT'}{' REFTAG'} || defined $tag->{'DEFAULT'}{'FEATURES'})
+        {
+            $fh->seek($base + $oScript + $tag->{' OFFSET'}, 0);
+            if (defined $tag->{'DEFAULT'}{' REFTAG'})
+            {
+                my ($ttag);
+                for ($ttag = $tag->{'DEFAULT'}{' REFTAG'}; defined $tag->{$ttag}{' REFTAG'}; $ttag = $tag->{$ttag}{' REFTAG'})
+                { }
+                $off = $tag->{$ttag}{' OFFSET'};
+            }
+            else
+            { $off = $tag->{'DEFAULT'}{' OFFSET'}; }
+            $fh->print(pack("n", $off));
+        }
+        $fh->seek($base + $oScript + $tag->{' OFFSET'} + 4, 0);
+        foreach (sort @{$tag->{'LANG_TAGS'}})
+        {
+            if (defined $tag->{$_}{' REFTAG'})
+            {
+                my ($ttag);
+                for ($ttag = $tag->{$_}{' REFTAG'}; defined $tag->{$ttag}{' REFTAG'}; $ttag = $tag->{$ttag}{' REFTAG'})
+                { }
+                $off = $tag->{$ttag}{' OFFSET'};
+            }
+            else
+            { $off = $tag->{$_}{' OFFSET'}; }
+            $fh->print(pack("a4n", $_, $off));
+        }
     }
     $fh->seek($base + $oScript + 2, 0);
     foreach $t (@script)
@@ -547,8 +595,18 @@ sub out
     foreach $t (@{$self->{'FEATURES'}{'FEAT_TAGS'}})
     {
         $tag = $self->{'FEATURES'}{$t};
-        $tag->{' OFFSET'} = tell($fh) - $base - $oFeat;
+        $oFtable = tell($fh) - $base - $oFeat;
+        $tag->{' OFFSET'} = $oFtable;
         $fh->print(pack("n*", 0, $#{$tag->{'LOOKUPS'}} + 1, @{$tag->{'LOOKUPS'}}));
+        if ($tag->{'PARMS'})
+        {
+            $end = $fh->tell();
+            $oParms = $end - $oFtable - $base - $oFeat;
+            $fh->seek($oFtable + $base + $oFeat,0);
+            $fh->print(pack("n",$oParms));
+            $fh->seek($end,0);
+            $tag->{'PARMS'}->out($fh);
+        }
     }
     $end = $fh->tell();
     $fh->seek($oFeat + $base + 2, 0);
@@ -559,23 +617,23 @@ sub out
     $fh->seek($end, 0);
     $oLook = $end - $base;
     
-    # Start Lookup List Table
+    # LookupListTable (including room for offsets to LookupTables)
     $nTags = $#{$self->{'LOOKUP'}} + 1;
     $fh->print(pack("n", $nTags));
     $fh->print(pack("n", 0) x $nTags);
-    $end = $fh->tell();		# end of LookupListTable = start of Lookups
+    $end = $fh->tell();     # end of LookupListTable = start of Lookups
     foreach $tag (@{$self->{'LOOKUP'}})
     { $nSubs += $self->num_sub($tag); }
     for ($i = 0; $i < $nTags; $i++)
     {
         $fh->seek($end, 0);
         $tag = $self->{'LOOKUP'}[$i];
-        $off = $end - $base - $oLook;	# BH 2004-03-04
+        $off = $end - $base - $oLook;   # BH 2004-03-04
         # Is there room, from the start of this i'th lookup, for this and the remaining
         # lookups to be wrapped in extension lookups?
         if (!defined $big && $off + ($nTags - $i) * 6 + $nSubs * 10 > 65535) # BH 2004-03-04
         {
-			# Not enough room -- need to start an extension!            
+            # Not enough room -- need to start an extension!            
             my ($k, $ext);
             $ext = $self->extension();
             # Must turn previous lookup into the first extension
@@ -589,10 +647,12 @@ sub out
             {
                 $tag = $self->{'LOOKUP'}[$j];
                 $nSub = $self->num_sub($tag);
+                $tag->{' OFFSET'} = $fh->tell() - $base - $oLook; # offset to this extension lookup
+                # LookupTable (including actual offsets to subtables)
                 $fh->print(pack("nnn", $ext, $tag->{'FLAG'}, $nSub));
-                $fh->print(pack("n*", map {$_ * 8 + 6 + $nSub * 2} (0 .. $nSub-1)));	# BH 2004-03-04
-                $tag->{' EXT_OFFSET'} = $fh->tell();	# = first extension lookup subtable
-                $tag->{' OFFSET'} = $tag->{' EXT_OFFSET'} - $nSub * 2 - 6 - $base - $oLook; # offset to this extension lookup
+                $fh->print(pack("n*", map {6 + $nSub * 2 + $_ * 8 + $tag->{'FLAG'} & 0x0010 ? 2 : 0 } (0 .. $nSub-1)));
+                $fh->print(pack("n", $tag->{'FILTER'})) if $tag->{'FLAG'} & 0x0010;
+                $tag->{' EXT_OFFSET'} = $fh->tell();    # = first extension lookup subtable
                 for ($k = 0; $k < $nSub; $k++)
                 { $fh->print(pack('nnN', 1, $tag->{'TYPE'}, 0)); }
             }
@@ -600,15 +660,17 @@ sub out
             $tag = $self->{'LOOKUP'}[$i];
             # Leave file positioned after all the extension lookups -- where the referenced lookups will start.
         }
-        $tag->{' OFFSET'} = $off unless defined $big;	# BH 2004-03-04
+        $tag->{' OFFSET'} = $off unless defined $big;   # BH 2004-03-04
         $nSub = $self->num_sub($tag);
         if (!defined $big)
         {
+            # LookupTable (including room for subtable offsets)
             $fh->print(pack("nnn", $tag->{'TYPE'}, $tag->{'FLAG'}, $nSub));
             $fh->print(pack("n", 0) x $nSub);
+            $fh->print(pack("n", $tag->{'FILTER'})) if $tag->{'FLAG'} & 0x0010;
         }
         else
-        { $end = $tag->{' EXT_OFFSET'}; }
+        { $end = $tag->{' EXT_OFFSET'}; } # Extension offsets computed relative to start of first Extension subtable -- corrected later
         my (@offs, $out, @refs);
         for ($j = 0; $j < $nSub; $j++)
         {
@@ -667,6 +729,173 @@ start of the subtable to be output
 sub out_sub
 { }
 
+=head2 $t->dirty
+
+Setting GPOS or GSUB dirty means that OS/2 may need updating, so set it dirty.
+
+=cut
+
+sub dirty
+{
+    my ($self, $val) = @_;
+    my $res = $self->SUPER::dirty ($val);
+    $self->{' PARENT'}{'OS/2'}->read->dirty($val) if exists $self->{' PARENT'}{'OS/2'};
+    $res;
+}
+
+=head2 $t->maxContext
+
+Returns the length of the longest opentype rule in this table.
+
+=cut
+
+sub maxContext
+{
+    my ($self) = @_;
+    
+    # Make sure table is read
+    $self->read;
+
+    # Calculate my contribution to OS/2 usMaxContext
+    
+    my ($maxcontext, $l, $s, $r, $m);
+   
+    for $l (@{$self->{'LOOKUP'}})        # Examine each lookup
+    {
+        for $s (@{$l->{'SUB'}})         # Multiple possible subtables for this lookup
+        {
+            for $r (@{$s->{'RULES'}})   # One ruleset for each covered glyph
+            {
+                for $m (@{$r})          # Multiple possible matches for this covered glyph 
+                {
+                    my $lgt;
+                    $lgt++ if exists $s->{'COVERAGE'};  # Count 1 for the coverage table if it exists
+                    for (qw(MATCH POST))                # only Input and Lookahead sequences count (Lookbehind doesn't) -- see OT spec.
+                    {
+                        $lgt += @{$m->{$_}} if exists $m->{$_};
+                    }
+                    $maxcontext = $lgt if $lgt > $maxcontext;
+                }
+            }
+            
+        }
+    }
+    
+    $maxcontext;    
+}    
+
+
+=head2 $t->update
+
+Perform various housekeeping items:
+
+For all lookups, set/clear 0x0010 bit of flag words based on 'FILTER' value.
+
+Sort COVERAGE table and RULES for all lookups.
+
+Unless $t->{' PARENT'}{' noharmony'} is true, update will make sure that GPOS and GSUB include 
+the same scripts and languages. Any added scripts and languages will have empty feature sets.
+
+=cut
+
+# Assumes we are called on both GSUB and GPOS. So simply ADDS scripts and languages to $self that it finds
+# in the other table.
+
+sub update
+{
+    my ($self) = @_;
+    
+    return undef unless ($self->SUPER::update);
+    
+    if (defined ($self->{'LOOKUP'}))
+    {
+            
+        # make flag word agree with mark filter setting:
+        for my $l (@{$self->{'LOOKUP'}})
+        {
+            if (defined $l->{'FILTER'})
+            { $l->{'FLAG'} |= 0x0010; }
+            else
+            { $l->{'FLAG'} &= ~0x0010; }
+        }
+        
+        unless ($Font::TTF::Coverage::dontsort)
+        {
+            # Sort coverage tables and rules of all lookups by glyphID
+            # The lookup types that need to be sorted are:
+            #    GSUB: 1.2 2 3 4 5.1 6.1 8    (However GSUB type 8 lookups are not yet supported by Font::TTF)
+            #    GPOS: 1.2 2.1 3 4 5 6 7.1 8.1
+            
+            for my $l (@{$self->{'LOOKUP'}})
+            {
+                next unless defined $l->{'SUB'};
+                for my $sub (@{$l->{'SUB'}})
+                {
+                    if (defined $sub->{'COVERAGE'} and $sub->{'COVERAGE'}{'cover'} and !$sub->{'COVERAGE'}{'dontsort'})
+                    {
+                        # OK! Found a lookup with coverage table:
+                        my @map = $sub->{'COVERAGE'}->sort();
+                        if (defined $sub->{'RULES'} and ($sub->{'MATCH_TYPE'} =~ /g/ or $sub->{'ACTION_TYPE'} =~ /[gvea]/))
+                        {
+                            # And also a RULES table which now needs to be re-sorted
+                            my $newrules = [];
+                            foreach (0 .. $#map)
+                            { push @{$newrules}, $sub->{'RULES'}[$map[$_]]; }
+                            $sub->{'RULES'} = $newrules;
+                        }
+                    }
+                        
+                    # Special case for Mark positioning -- need to also sort the MarkArray
+                    if (exists($sub->{'MARKS'}) and ref($sub->{'MATCH'}[0]) =~ /Cover/ and $sub->{'MATCH'}[0]{'cover'} and !$sub->{'MATCH'}[0]{'dontsort'})
+                    {
+                        my @map = $sub->{'MATCH'}[0]->sort();
+                        my $newmarks = [];
+                        foreach (0 .. $#map)
+                        { push @{$newmarks}, $sub->{'MARKS'}[$map[$_]]; }
+                        $sub->{'MARKS'} = $newmarks;
+                    }
+                }
+            }
+        }
+    }
+
+    # Enforce script/lang congruence unless asked not to:
+    return $self if $self->{' PARENT'}{' noharmony'};
+
+    # Find my sibling (GSUB or GPOS, depending on which I am)
+    my $sibling = ref($self) eq 'Font::TTF::GSUB' ? 'GPOS' : ref($self) eq 'Font::TTF::GPOS' ? 'GSUB' : undef;
+    return $self unless $sibling && defined $self->{' PARENT'}{$sibling};
+    $sibling = $self->{' PARENT'}{$sibling};
+    
+    # Look through scripts defined in sibling:
+    for my $sTag (grep {length($_) == 4} keys %{$sibling->{'SCRIPTS'}})
+    {
+        my $sibScript = $sibling->{'SCRIPTS'}{$sTag};
+        $sibScript = $sibling->{$sibScript->{' REFTAG'}} if exists $sibScript->{' REFTAG'} && $sibScript->{' REFTAG'} ne '';
+        
+        $self->{'SCRIPTS'}{$sTag} = {} unless defined $self->{'SCRIPTS'}{$sTag}; # Create script if not present in $self
+        
+        my $myScript = $self->{'SCRIPTS'}{$sTag};
+        $myScript = $self->{$myScript->{' REFTAG'}} if exists $myScript->{' REFTAG'} && $myScript->{' REFTAG'} ne '';
+                
+        foreach my $lTag (@{$sibScript->{'LANG_TAGS'}})
+        {
+            # Ok, found a script/lang that is in our sibling.
+            next if exists $myScript->{$lTag};  # Already in $self
+            
+            # Need to create this lang:
+            push @{$myScript->{'LANG_TAGS'}}, $lTag;
+            $myScript->{$lTag} = { 'FEATURES' => [] };
+        }
+
+        if (defined $sibScript->{'DEFAULT'} && !defined $myScript->{'DEFAULT'})
+        {
+            # Create default lang for this script.
+            $myScript->{'DEFAULT'} = { 'FEATURES' => [] };
+        }
+    }
+    $self;
+}
 
 =head1 Internal Functions & Methods
 
@@ -694,7 +923,7 @@ sub copy
 =head2 $t->read_cover($cover_offset, $lookup_loc, $lookup, $fh, $is_cover)
 
 Reads a coverage table and stores the results in $lookup->{' CACHE'}, that is, if
-it hasn't been read already.
+it has not been read already.
 
 =cut
 
@@ -715,7 +944,7 @@ sub read_cover
 }
 
 
-=head2 ref_cache($obj, $cache, $offset)
+=head2 ref_cache($obj, $cache, $offset [, $template])
 
 Internal function to keep track of the local positioning of subobjects such as
 coverage and class definition tables, and their offsets.
@@ -731,11 +960,13 @@ Uses tricks for Tie::Refhash
 
 sub ref_cache
 {
-    my ($obj, $cache, $offset) = @_;
+    my ($obj, $cache, $offset, $template) = @_;
 
     return 0 unless defined $obj;
-    $cache->{"$obj"}[0] = $obj unless defined $cache->{"$obj"};
-    push (@{$cache->{"$obj"}[1]}, $offset);
+    $template ||= 'n';
+    unless (defined $cache->{"$obj"})
+    { push (@{$cache->{''}}, $obj); }
+    push (@{$cache->{"$obj"}}, [$offset, $template]);
     return 0;
 }
 
@@ -766,12 +997,12 @@ sub out_final
     foreach $r (@$cache_list)
     {
         $offs = $r->[1];
-        foreach $t (sort keys %{$r->[0]})
+        foreach $t (@{$r->[0]{''}})
         {
             $str = "$t";
             if (!defined $master_cache->{$str})
             {
-                my ($vec) = $r->[0]{$str}[0]->signature();
+                my ($vec) = $t->signature();
                 if ($vecs{$vec})
                 { $master_cache->{$str} = $master_cache->{$vecs{$vec}}; }
                 else
@@ -780,13 +1011,16 @@ sub out_final
                     $master_cache->{$str} = ($state ? length($out) : $fh->tell())
                                                                        - $base_loc;
                     if ($state)
-                    { $out .= $r->[0]{$str}[0]->out($fh, 1); }
+                    { $out .= $t->out($fh, 1); }
                     else
-                    { $r->[0]{$str}[0]->out($fh, 0); }
+                    { $t->out($fh, 0); }
                 }
             }
-            foreach $s (@{$r->[0]{$str}[1]})
-            { substr($out, $s, 2) = pack('n', $master_cache->{$str} - $offs); }
+            foreach (@{$r->[0]{$str}})
+            {
+                $s = pack($_->[1], $master_cache->{$str} - $offs);
+                substr($out, $_->[0], length($s)) = $s;
+            }
         }
     }
     if ($state)
@@ -1060,7 +1294,7 @@ sub out_context
     } elsif ($type == 6 && $fmt == 3)
     {
         $r = $lookup->{'RULES'}[0][0];
-		no strict 'refs';	# temp fix - more code needed (probably "if" statements in the event 'PRE' or 'POST' are empty)
+        no strict 'refs';   # temp fix - more code needed (probably "if" statements in the event 'PRE' or 'POST' are empty)
         $out .= pack('n2', $fmt, defined $r->{'PRE'} ? scalar @{$r->{'PRE'}} : 0);
         foreach $t (@{$r->{'PRE'}})
         { $out .= pack('n', Font::TTF::Ttopen::ref_cache($t, $ctables, length($out) + $base)); }
@@ -1077,6 +1311,9 @@ sub out_context
     $out;
 }
 
+1;
+
+
 =head1 BUGS
 
 =over 4
@@ -1091,10 +1328,17 @@ repeated if necessary. Within lookup sharing is possible.
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
-1;
-
