@@ -2,7 +2,7 @@ package Font::TTF::Vhea;
 
 =head1 NAME
 
-TTF:Vhea - Vertical Header table
+Font::TTF::Vhea - Vertical Header table
 
 =head1 DESCRIPTION
 
@@ -29,19 +29,32 @@ This is a simple table with just standards specified instance variables
 =cut
 
 use strict;
-use vars qw(@ISA %fields);
+use vars qw(@ISA %fields @field_info);
 
 require Font::TTF::Table;
 use Font::TTF::Utils;
 
 @ISA = qw(Font::TTF::Table);
+@field_info = (
+    'version' => 'v',
+    'Ascender' => 's',
+    'Descender' => 's',
+    'LineGap' => 's',
+    'advanceHeightMax' => 'S',
+    'minTopSideBearing' => 's',
+    'minBottomSideBearing' => 's',
+    'yMaxExtent' => 's',
+    'caretSlopeRise' => 's',
+    'caretSlopeRun' => 's',
+    'metricDataFormat' => '+10s',
+    'numberOfVMetrics' => 's');
 
 sub init
 {
-    my ($k, $v, $c);
-    while (<Font::TTF::Vhea::DATA>)
+    my ($k, $v, $c, $i);
+    for ($i = 0; $i < $#field_info; $i += 2)
     {
-        ($k, $v, $c) = TTF_Init_Fields($_, $c);
+        ($k, $v, $c) = TTF_Init_Fields($field_info[$i], $c, $field_info[$i + 1]);
         next unless defined $k && $k ne "";
         $fields{$k} = $v;
     }
@@ -85,6 +98,18 @@ sub out
     $self;
 }
 
+=head2 $t->minsize()
+
+Returns the minimum size this table can be. If it is smaller than this, then the table
+must be bad and should be deleted or whatever.
+
+=cut
+
+sub minsize
+{
+    return 36;
+}
+
 
 =head2 $t->update
 
@@ -101,14 +126,13 @@ sub update
     my ($num);
     my ($i, $maw, $mlsb, $mrsb, $mext, $aw, $lsb, $ext);
 
+    return undef unless ($self->SUPER::update);
     return undef unless (defined $vmtx && defined $self->{' PARENT'}{'loca'});
     $vmtx->read->update;
     $self->{' PARENT'}{'loca'}->read->update;
     $glyphs = $self->{' PARENT'}{'loca'}{'glyphs'};
     $num = $self->{' PARENT'}{'maxp'}{'numGlyphs'};
 
-    return undef unless ($vmtx->{' isDirty'} || $self->{' PARENT'}{'loca'}{' isDirty'});
-    
     for ($i = 0; $i < $num; $i++)
     {
         $aw = $vmtx->{'advance'}[$i];
@@ -127,8 +151,6 @@ sub update
     $self->{'minBottomSideBearing'} = $mrsb;
     $self->{'yMaxExtent'} = $mext;
     $self->{'numberOfVMetrics'} = $vmtx->numMetrics;
-
-    $self->{' isdirty'} = 1;
     $self;
 }
 
@@ -142,23 +164,18 @@ None known
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
 
-__DATA__
-version, f
-Ascender, s
-Descender, s
-LineGap, s
-advanceHeightMax, S
-minTopSideBearing, s
-minBottomSideBearing, s
-yMaxExtent, s
-caretSlopeRise, s
-caretSlopeRun, s
-metricDataFormat, +10s
-numberOfVMetrics, s
-
