@@ -51,7 +51,7 @@ sub read
     my ($numh, $numg);
 
     $numh = $self->{' PARENT'}{'vhea'}->read->{'numberOfVMetrics'};
-    $numg = $self->{' PARENT'}{'maxp'}->read->{'numGlyphs'};
+    $numg = $self->{' PARENT'}{'maxp'}{'numGlyphs'};
     $self->_read($numg, $numh, "advance", "top");
 }
 
@@ -79,8 +79,18 @@ None known
 
 =head1 AUTHOR
 
-Martin Hosken Martin_Hosken@sil.org. See L<Font::TTF::Font> for copyright and
-licensing.
+Martin Hosken L<Martin_Hosken@sil.org>. 
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
 
 =cut
 
+
