@@ -43,3 +43,19 @@ require Font::TTF::Table;
 @ISA = qw(Font::TTF::Table);
 
 1;
+
+=head1 AUTHOR
+
+Bob Hallissy. L<Bob_Hallissy@sil.org>
+
+
+=head1 LICENSING
+
+Copyright (c) 1998-2013, SIL International (http://www.sil.org) 
+
+This module is released under the terms of the Artistic License 2.0. 
+For details, see the full text of the license in the file LICENSE.
+
+The test suite contains test fonts released under the SIL Open Font License 1.1, see OFL.txt.
+
+=cut
\ No newline at end of file
